/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is from autoform.
 *
 * The Initial Developer of the Original Code is
 * Michael Kessler, Bluewin AG <michael.kessler@team.bluewin.com>.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
//************************************************/
// preference class
//************************************************/

function preferences() {
	
   // public properties
   this.loadNeverArray = null;
   this.loadAlwaysArray = null;
   this.saveNeverArray = null;
   this.saveAlwaysArray = null;
   this.excludeParametersArray = null;
   
   this.autouser = null;
   this.autoload = false;
   this.autosave = false;
   this.visualize = true;
   this.overwrite = true;
   this.emptyvalues = false;
   this.savepasswords = true;
   this.encryptpasswords = true;
   
   // private properties
   var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
   var nullPrefs = prefService.getBranch(null);
   var autoPrefs = prefService.getBranch("autoform.");
   var userPrefs = prefService.getBranch("autoform.autoform-users.");
  
   var prefCount = {
      value : 0
   };
      
   var prefArray = userPrefs.getChildList("", prefCount);
   if (prefArray.length == 0) {
						userPrefs.setCharPref("user0", doHexEncode("Default"));   			
						this.autouser = "user0";
   }
   
   var virtualUserPrefs;
   var hashhostname = true;
   var hashpathname = true;
   var hashsearch = true;
   var hashform = true;
   
   var sbSrv = Components.classes['@mozilla.org/intl/stringbundle;1'].createInstance();
   var sbSrvI = sbSrv.QueryInterface(Components.interfaces.nsIStringBundleService);
   var autoformBundle = sbSrvI.createBundle("chrome://autoform/locale/autoform.properties");
   
   // define public methods
   this.doConfigure = doConfigure;
   this.doUserChange = doUserChange;
   this.doOpenDialog = doOpenDialog;
   
   // preference observer
   var prefChangeObserver = {
      observe :
      
      function(aSubject, aTopic, aData) {
         doConfigure();
      }
   };
   
   // add observer to preference
   var qi = nullPrefs.QueryInterface(Components.interfaces["nsIPrefBranchInternal"]);
   qi.addObserver("autoform", prefChangeObserver, false);

   //************************************************/
   // read preferences and configure application
   //************************************************/
   
   function doConfigure() {
   	  try {
			      // autoform user
			      this.autouser = getCharPrefFromMain("autouser", "user0");
			      virtualUserPrefs = prefService.getBranch("autoform." + this.autouser + ".");
									
			      // automation settings
			      this.autoload = getBoolPref("autoload", false);
			      this.autosave = getBoolPref("autosave", false);
			
									if (getBoolPref("mozillaformfill", false)) {
									   nullPrefs.setBoolPref("browser.formfill.enable", false);
									} else {
									   nullPrefs.setBoolPref("browser.formfill.enable", true);
									}
									
			      // action settings
			      this.visualize = getBoolPref("visualize", true);
			      this.overwrite = getBoolPref("overwrite", true);
			      this.emptyvalues = getBoolPref("emptyvalues", false);
			      
			      // password setting
			      this.savepasswords = getBoolPref("savepasswords", true);
			      this.encryptpasswords = getBoolPref("encryptpasswords", true);
			      
									if (getBoolPref("mozillapasswords", false)) {
									   nullPrefs.setBoolPref("signon.rememberSignons", false);
									} else {
									   nullPrefs.setBoolPref("signon.rememberSignons", true);
									}
			
			      // context settings
			      this.hashhostname = getBoolPref("hashhostname", true);
			      this.hashpathname = getBoolPref("hashpathname", true);
			      this.hashsearch = getBoolPref("hashsearch", true);
			      this.hashform = getBoolPref("hashform", true);
			      
			      // customize toolbar
			      doElementVisible("autoform-button-load-form", getBoolPref("barloadform", true));
			      doElementVisible("autoform-button-save-form", getBoolPref("barsaveform", true));
			      doElementVisible("autoform-button-reset-form", getBoolPref("barresetform", true));
			      doElementVisible("autoform-button-delete-form", getBoolPref("bardeleteform", true));
			      doElementVisible("autoform-button-config", getBoolPref("barconfig", true));
			      doElementVisible("autoform-button-user-select", getBoolPref("baruserselect", false));
			      doElementVisible("autoform-button-help", getBoolPref("barhelp", true));
			      
			      // load autoload/autosave lists
			      this.loadNeverArray = getBranchAsArray("autoform-loadNever");
			      this.loadAlwaysArray = getBranchAsArray("autoform-loadAlways");
			      this.saveNeverArray = getBranchAsArray("autoform-saveNever");
			      this.saveAlwaysArray = getBranchAsArray("autoform-saveAlways");
			      this.excludeParametersArray = getBranchAsArray("autoform-excludeParameters");
			      
			      // set menu style
			      barstyle = getCharPref("barstyle", "icontext");
			      
			      if(barstyle == "icon") {
			         doToolbarIcons(true);
			         doToolbarText(false);
			      }
			      else if(barstyle == "text") {
			         doToolbarIcons(false);
			         doToolbarText(true);
			      }
			      else if(barstyle == "icontext") {
			         doToolbarIcons(true);
			         doToolbarText(true);
			      }
			      
			      // configure users
			      doUserSettings();

   		 } catch(e) {
   		    debug(e, "doConfigure()");
   		 } 			      
   }
   
   //************************************************/
   // check user settings, must have at least one default user
   //************************************************/
   
   function doUserSettings() {
      try {
			      // get users
			      var prefCount = {
			         value : 0
			      };
			      
			      var prefArray = userPrefs.getChildList("", prefCount);
			      this.autouser = getCharPrefFromMain("autouser", "user0");
			      
			      // get menupop
			      var userpop = document.getElementById("autoform-button-user-select-pop");
			
				     // add new users to menu popup 
				     var menuitem, userElement;
				
				     for(i = 0; i < prefCount.value; i++) {
				        userElement = document.getElementById(prefArray[i]);
				
				        if(userElement == null) {
				           // create new user
				           menuitem = document.createElement("menuitem");
				           menuitem.setAttribute("label", doHexDecode(userPrefs.getCharPref(prefArray[i])));
				           menuitem.setAttribute("id", prefArray[i]);
				           menuitem.setAttribute("value", prefArray[i]);
				           menuitem.setAttribute("type", "radio");
				           menuitem.setAttribute("oncommand", "actions.doSave(); preferences.doUserChange(this); actions.doLoad();");
				           if(prefArray[i] == this.autouser) {
				              menuitem.setAttribute("checked", "true");
				           }
				           userpop.appendChild(menuitem);
				        }
				     }
				     
				     // remove not used users from menu popup
						      var userpop = document.getElementById("autoform-button-user-select-pop");
						      
				     for(c = 0; c < userpop.childNodes.length; c++) {
				     			var id = userpop.childNodes[c].getAttribute("id");
				     			
				     			var found = false;
				     			
				     			for(i = 0; i < prefCount.value; i++) {
				     						if (prefArray[i] == id) {
				     								found = true;
				     						}
				     			}
				     			
				     			if (!found) {
				     					userpop.removeChild(userpop.childNodes[c]);
				     			}
			      }
   		 } catch(e) {
   		    debug(e, "doUserSettings()");
   		 } 			      

   }

   //************************************************/
   // change user
   //************************************************/
   
   function doUserChange(menu) {
   	  try {
			      autoPrefs.setCharPref("autouser", menu.value);
			      this.autouser=menu.value;
			      doConfigure();
   		 } catch(e) {
   		    debug(e, "doUserChange()");
   		 } 			      
   }

   //************************************************/
   // show/hide toolbar icons
   //************************************************/
   
   function doToolbarIcons(visible) {
						try {
			      var imagestatus;
			
			      if(visible) {
			         imagestatus = "on";
			      }
			      else {
			         imagestatus = "off";
			      }
			
			      document.getElementById("autoform-button-load-form").setAttribute("status", imagestatus);
			      document.getElementById("autoform-button-save-form").setAttribute("status", imagestatus);
			      document.getElementById("autoform-button-reset-form").setAttribute("status", imagestatus);
			      document.getElementById("autoform-button-delete-form").setAttribute("status", imagestatus);
			      document.getElementById("autoform-button-config").setAttribute("status", imagestatus);
			      document.getElementById("autoform-button-user-select").setAttribute("status", imagestatus);
			      document.getElementById("autoform-button-help").setAttribute("status", imagestatus);
			      
   		 } catch(e) {
   		    debug(e, "doToolbarIcons()");
   		 } 			      
   }

   //************************************************/
   // show/hide toolbar text
   //************************************************/
   
   function doToolbarText(visible) {
						try {	
			   			this.autouser = getCharPrefFromMain("autouser", "user0");
			   			
			      if(visible) {
			         document.getElementById("autoform-button-load-form").label = autoformBundle.GetStringFromName("load");
			         document.getElementById("autoform-button-save-form").label = autoformBundle.GetStringFromName("save");
			         document.getElementById("autoform-button-reset-form").label = autoformBundle.GetStringFromName("reset");
			         document.getElementById("autoform-button-delete-form").label = autoformBundle.GetStringFromName("delete");
			         document.getElementById("autoform-button-config").label = autoformBundle.GetStringFromName("config");
			         document.getElementById("autoform-button-user-select").label = autoformBundle.GetStringFromName("user") + ": " + doHexDecode(userPrefs.getCharPref(this.autouser));
			         document.getElementById("autoform-button-help").label = autoformBundle.GetStringFromName("help");
			      }
			      else {
			         document.getElementById("autoform-button-load-form").label = "";
			         document.getElementById("autoform-button-save-form").label = "";
			         document.getElementById("autoform-button-reset-form").label = "";
			         document.getElementById("autoform-button-delete-form").label = "";
			         document.getElementById("autoform-button-config").label = "";
			         document.getElementById("autoform-button-user-select").label = "";
			         document.getElementById("autoform-button-help").label = "";
			      }
   		 } catch(e) {
   		    debug(e, "doToolbarText()");
   		 } 			      

   }

   //************************************************/
   // set element visibility
   //************************************************/
   
   function doElementVisible(name, visible) {
   	  try {
			      if(visible) {
			         document.getElementById(name).hidden = false;
			      }
			      else {
			         document.getElementById(name).hidden = true;
			      }
      
   		 } catch(e) {
   		    debug(e, "doElementVisible");
   		 } 			      
   }

   //************************************************/
   // open preference dialog
   //************************************************/
   
   function doOpenDialog() {
   	  try {
			      window.open('chrome://autoform/content/preference-dialog.xul', 'pref', 'chrome,dialog,centerscreen,modal');
   		 } catch(e) {
   		    debug(e, "doOpenDialog()");
   		 } 			      

   }

   //*********************************************************/
   // read virtual user boolean preference value, return default value if not found
   //*********************************************************/
   
   function getBoolPref(name, defaultvalue) {
   	  try {
			      if(virtualUserPrefs.prefHasUserValue(name)) {
			         return virtualUserPrefs.getBoolPref(name);
			      }
			
			      virtualUserPrefs.setBoolPref(name, defaultvalue);
			      return defaultvalue;

   		 } catch(e) {
   		    debug(e, "getBoolPref()");
   		 } 			      
   }

   //************************************************/
   // read virtual user char preference value, return default value if not found
   //************************************************/
   
   function getCharPref(name, defaultvalue) {
   	  try {
			      if(virtualUserPrefs.prefHasUserValue(name)) {
			         return virtualUserPrefs.getCharPref(name);
			      }
			
			      virtualUserPrefs.setCharPref(name, defaultvalue);
			      return defaultvalue;
      
   		 } catch(e) {
   		    debug(e, "getCharPref()");
   		 }        
   }

   //************************************************/
   // read main char preference value, return default value if not found
   //************************************************/
   
   function getCharPrefFromMain(name, defaultvalue) {
   	  try {
			      if(autoPrefs.prefHasUserValue(name)) {
			         return autoPrefs.getCharPref(name);
			      }
			
			      autoPrefs.setCharPref(name, defaultvalue);
			      return defaultvalue;

   		 } catch(e) {
   		    debug(e, "getCharPrefFromMain()");
   		 }  
   }

   //************************************************/
   // fill the values of a preference branch into an array
   //************************************************/
   
   function getBranchAsArray(name) {

					 try {
									autouser = getCharPrefFromMain("autouser", "user0");
									
			      // get branch
			      var listPrefs = prefService.getBranch("autoform." + autouser + "." + name + ".");
			      var prefCount = {
			         value : 0
			      };
			
			      var prefArray = listPrefs.getChildList("", prefCount);
			
			      // fill array with branch values
			      var theArray = null;
			
			      if(prefCount.value != 0) {
			         var theArray = new Array(prefCount);
			         for(i = 0; i < prefCount.value; i++) {
			            theArray[i] = doHexDecode(listPrefs.getCharPref(prefArray[i]));
			         }
			      }
			
			      return theArray;
			      
   		 } catch(e) {
   		    debug(e, "getBranchAsArray()");
   		 }  
   }

			//************************************************/
			// Decodes the String to Hex values
			//************************************************/
			   
			function _to_int(x1) {
				  try {
			      return x1 >= 48 && x1 < 58 ? x1 - 48 : x1 - 97 + 10;

   		 } catch(e) {
   		    debug(e, "_to_int()");
   		 }  
			}
			   
			function doHexDecode(s) {
				  try {
						   var r = "";
						   for(var i = 2; i < s.length; i += 4) {
						      x1 = _to_int(s.charCodeAt(i));
						      x2 = _to_int(s.charCodeAt(i + 1));
						      x3 = _to_int(s.charCodeAt(i + 2));
						      x4 = _to_int(s.charCodeAt(i + 3));
						      r += String.fromCharCode(((x1 << 12) & 0xF000) |((x2 << 8) & 0x0F00) |((x3 << 4) & 0x00F0) |(x4 & 0x000F));
						   }
						   return r;
						   
   		 } catch(e) {
   		    debug(e, "doHexDecode()");
   		 }  						   
			}   
			
   //************************************************/
   // Debug messages
   //************************************************/

   function debug(text, method) {
   			dump("AutoForm error in preferences.js method " + method + ": " + text);
   }
}
